//using System;
using System.Collections.Generic;
using System.Text;
//using System.Threading;
using System.Windows.Forms;
using MediaPortal.GUI.Library;
//using MediaPortal.Util;
using MediaPortal.Player;
using MediaPortal.Configuration;
using MediaPortal.Dialogs;
using System.Runtime.InteropServices;
using System.Diagnostics;

namespace eventGhostTrigger
{
  public class eventGhostTrigger : IPlugin, ISetupForm
  {
    public const string PLUGIN_NAME = "eventGhost Trigger";
    public string egPath;
    public string part2;
    public string part3;
    public bool payload;
    ProcessStartInfo startInfo = new ProcessStartInfo();

    public eventGhostTrigger()
    {
      using (MediaPortal.Profile.Settings xmlReader = new MediaPortal.Profile.Settings(Config.GetFile(Config.Dir.Config, "MediaPortal.xml")))
      {
        egPath = xmlReader.GetValueAsString(eventGhostTrigger.PLUGIN_NAME, "egPath", "");
        part2 = xmlReader.GetValueAsString(eventGhostTrigger.PLUGIN_NAME, "part2", "Type");
        part3 = xmlReader.GetValueAsString(eventGhostTrigger.PLUGIN_NAME, "part3", "Status");
        payload = xmlReader.GetValueAsBool(eventGhostTrigger.PLUGIN_NAME, "payload", false);
      }
      startInfo.CreateNoWindow = false;
      startInfo.UseShellExecute = false;
      startInfo.FileName = @egPath + @"\EventGhost.exe";
      //startInfo.FileName = @"c:\Utilities\EventGhost\EventGhost.exe";
      startInfo.WindowStyle = ProcessWindowStyle.Hidden;

      SetLevelForPlayback("Plugin", "Start");
    }
    ~eventGhostTrigger()
    {
      SetLevelForPlayback("Plugin", "Stop");
    }

    #region ISetupForm Members

    public string Author()
    {
      return "Yoav Mor, Dragy for 1.2 compatibility";
    }

    public bool CanEnable()
    {
      return true;
    }

    public bool DefaultEnabled()
    {
      return true;
    }

    public string Description()
    {
      return "Send events to eventGhost";
    }

    public bool GetHome(out string strButtonText, out string strButtonImage, out string strButtonImageFocus, out string strPictureImage)
    {
      strButtonText = strButtonImage = strButtonImageFocus = strPictureImage = null;
      return false;
    }

    public int GetWindowId()
    {
      return 9135;
    }

    public bool HasSetup()
    {
      return true;
    }

    public string PluginName()
    {
      return PLUGIN_NAME;
    }

    public void ShowPlugin()
    {
      FormSettings frm = new FormSettings();
      frm.ShowDialog();
    }

    #endregion


    #region IPlugin Members
    void IPlugin.Start()
    {
      WriteToLog("Attaching Listeners");
      // Since we have found a device we can start listening for playback event
      g_Player.PlayBackStarted += new g_Player.StartedHandler(OnVideoStarted);
      g_Player.PlayBackEnded += new g_Player.EndedHandler(OnVideoEnded);
      g_Player.PlayBackStopped += new g_Player.StoppedHandler(OnVideoStopped);
      GUIWindowManager.OnNewAction += new OnActionHandler(GUIWindowManager_OnNewAction);
    }
    void IPlugin.Stop()
    {
      WriteToLog("Stopping");
    }
    #endregion

    #region Playback events
    public void OnVideoStarted(g_Player.MediaType type, string s)
    {

      WriteToLog("Started " + type);
      SetLevelForPlayback(type.ToString(), "Play");
    }

    public void OnVideoEnded(g_Player.MediaType type, string s)
    {
      WriteToLog("Ended " + type);
      SetLevelForPlayback(type.ToString(), "End");
    }

    public void OnVideoStopped(g_Player.MediaType type, int i, string s)
    {
      WriteToLog("Stopped " + type);
      WriteToLog("Returning to level before play");
      SetLevelForPlayback(GetCurrentMediaType(), "Stop");
    }

    void GUIWindowManager_OnNewAction(Action action)
    {
      //WriteToLog("New Action");
      if (action.wID == Action.ActionType.ACTION_PAUSE)
      {
        WriteToLog("New Action: Paused");
        if (g_Player.Paused)
        {
          WriteToLog("Paused");
          SetLevelForPlayback(GetCurrentMediaType(), "Pause");
        }
        else
        {
          WriteToLog("New Action: Resumed");
          WriteToLog("Resumed");
          SetLevelForPlayback(GetCurrentMediaType(), "Resume");
        }
      }
      else if (action.wID == Action.ActionType.ACTION_PLAY || action.wID == Action.ActionType.ACTION_MUSIC_PLAY)
      {
        if (g_Player.Playing || g_Player.Paused)
        {
          WriteToLog("New Action: Play, but already playing");
        }
        else
        {
          WriteToLog("New Action: Play");
          WriteToLog("Play");
        }
        SetLevelForPlayback(GetCurrentMediaType(), "Play");
      }
    }
    #endregion


    private string GetCurrentMediaType()
    {
      if (g_Player.IsMusic)
      {
        WriteToLog("Detect MediaType: Music");
        return g_Player.MediaType.Music.ToString();
      }
      else if (g_Player.IsRadio)
      {
        WriteToLog("Detect MediaType: Radio");
        return g_Player.MediaType.Radio.ToString();
      }
      else if (g_Player.IsTVRecording)
      {
        WriteToLog("Detect MediaType: Recording");
        return g_Player.MediaType.Recording.ToString();
      }
      else if (g_Player.IsTV)
      {
        WriteToLog("Detect MediaType: TV");
        return g_Player.MediaType.TV.ToString();
      }
      else
      {
        WriteToLog("Detect MediaType: Video");
        return g_Player.MediaType.Video.ToString();
      }
    }

    private void SetLevelForPlayback(string mediaType, string Level)
    {
      string part2String;
      string part3String;
      string middleString;
      if (part2.Equals("Type"))
        part2String = mediaType;
      else
        part2String = Level;

      if (part3.Equals("Type"))
        part3String = mediaType;
      else
        part3String = Level;

      if (payload)
        middleString = " ";
      else
        middleString = ".";

      startInfo.Arguments = "-e MediaPortal." + part2String + middleString + part3String;
      // Start the process.
      try
      {
        // Start the process with the info we specified.
        // Call WaitForExit and then the using statement will close.
        using (Process exeProcess = Process.Start(startInfo))
        {
          exeProcess.WaitForExit();
        }
      }
      catch
      {
        //MessageBox.Show(egPath + " not a valid path for EventGhost, run setup again");
        // Log error.
      }

    }

    private void WriteToLog(string textToLog)
    {
      Log.Debug(PLUGIN_NAME + "(process): " + textToLog);
    }
  }


}
